////////////////////////////////////////////////////////////////////////////////
// copyright 2017 Phenotype Screening Corporation, and
// distributed under the terms of the GNU General Public License
/*  This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
// read_txt_leaf.c
// see below for gcc compile line
// read RGB raw image file and write out a 16 bit raw image
// do color thresholding 
// RGB values of exemplar set on command line
// divergence from exemplar set on command line
// minimum light intensity set on command line - see shell script
// RGB values that deviate in color from exemplar 
// more than THETA thresh are set to 0 in output file
// compile lines below
// gcc -Wall -o read_txt_leaf read_txt_leaf.c -lm # for debug
// gcc -O3 -o read_txt_leaf read_txt_leaf.c -lm  # for optimize
// how to call executable  - see shell script
// note './' in front of executable required in Linux
// ./read_txt_leaf R G B THETA THRESHOLD COLS ROWS BASEFILE
// ./read_txt_leaf 80 106 41 0.125 75 3000 2000 P8241314

#include <stdio.h>   
#include <stdlib.h>
#include <string.h>
#include <math.h>

// prototypes
float find_angle(int a[3],int b[3],float *projection);
void on_error(char message[], int code);

////////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
    char input_base[300];
    char input_file_name[300];   /* input file */
    char output_file_name[300];   /* input file */
    char color_val[100];
    char dimension_val[100];
    char max_angle_val[100];
    float max_angle;
    FILE *infile;
    FILE *outfile;
    int r, c;
    int row, col;
    int no_read;                // how many entries written or read
    short unsigned int tmp_val;
    int a[3], b[3]; // a is exemplar, b is rgb of pixel of interest
    float projection;  // projection of b[] vector on a[] exemplar vector 
    unsigned char b_char[3];
    float theta;  // angle between 
    char thresh_val[100];  // lowest allowable value of light intensity
    char thresh;
    
    if(argc!=9){
	    printf("\nusage: ./read_txt_leaf red green blue theta thresh r g b basefilename\n");
	    exit(1);
    }
    strcpy(color_val,argv[1]);
    a[0] = atoi(color_val); 
    printf("\na[0] = %d",a[0]);
    strcpy(color_val,argv[2]);
    a[1] = atoi(color_val); 
    printf("\na[1] = %d",a[1]);
    strcpy(color_val,argv[3]);
    a[2] = atoi(color_val); 
    printf("\na[2] = %d",a[2]);
    strcpy(max_angle_val,argv[4]);
    sscanf(max_angle_val,"%f", &max_angle);
    printf("\nmax_angle = %f",max_angle); 
    strcpy(thresh_val,argv[5]);
    thresh = atoi(thresh_val);
    printf("\nthresh = %d",thresh);
    strcpy(input_base,argv[6]);
    printf("\ninput_base = %s",input_base);
    strcpy(dimension_val,argv[7]);
    row = atoi(dimension_val); 
    strcpy(dimension_val,argv[8]);
    col = atoi(dimension_val); 
    
    strcpy(input_file_name, input_base);
    strcat(input_file_name,".rgb"); // this file rgb specific
    printf("\ninput_file_name = %s",input_file_name);
    strcpy(output_file_name, input_base);
    strcat(output_file_name,".raw");
    printf("\noutput_file_name = %s\n",output_file_name);
    // open output file fpr writing
    if ((outfile = fopen(output_file_name, "wb")) == NULL) {
        on_error("in fprint_short_int_mat, cannot open data output file ",1);
    }
    // open input file for reading
    if ((infile = fopen(input_file_name, "rb")) == NULL) {
        on_error("in main, cannot open data input file ",1);
    }
    for(r=0;r<row;r++){
        for(c=0;c<col;c++){
            no_read = fread(b_char, sizeof(char), 3,infile);
			if (no_read != 3) {
                    on_error("error reading input file, probably file size error",1);
            }            
            b[0] = (int)b_char[0];
            b[1] = (int)b_char[1];
            b[2] = (int)b_char[2];
            // call to dot product function
            theta = find_angle(a,b,&projection);
            //printf("\nprojection = %f",projection);
            //printf("\ntheta = %f",theta);
            // adjust pixel values and write projection of pixel rgb 
            // onto exemplar val out to file
            if(theta<max_angle&&projection>thresh){  
                tmp_val = (unsigned short int)projection;
                no_read = fwrite(&tmp_val, 
                    sizeof(unsigned short int),1, outfile);
                if (no_read != 1) {
                    on_error("output file does not contain correct number of pixels",1);
                }
            }
            else{                          // color too far away from exemplar
                tmp_val = (unsigned short int)0;
                no_read = fwrite(&tmp_val, 
                     sizeof(unsigned short int),1, outfile);
                if (no_read != 1) {
                    on_error("output file does not contain correct number of pixels",1);
                }
            }      
        }    
    }
    fclose(outfile);
    return 1;
}

////////////////////////////////////////////////////////////////////////////////
// find_angle()
// copyright 2016 Phenotype Screening Corporation. All rights reserved.
// input is two 3 dimension vectors
// use dot product of two vectors to find angle
// output is angle between vectors in radians
////////////////////////////////////////////////////////////////////////////////
float find_angle(
    int a[3],
    int b[3],
    float *projection
){

    float a_abs;
    float b_abs;
    float dot_prod;
    float cos_theta;
    float theta; // radians
    int j;
    
    //printf("\na = %d, %d, %d b = %d, %d, %d",a[0],a[1],a[2],b[0],b[1],b[2]);
    
    dot_prod = 0.0;
    for(j=0;j<3;j++){
        dot_prod += (float)a[j]*(float)b[j];    
        //printf("\nj = %d dot_prod = %f",j,dot_prod);
    }
    //printf("\ndot_prod = %f",dot_prod);
     
    a_abs = 0.0;
    for(j=0;j<3;j++){
        a_abs += (float)a[j]*(float)a[j];
    }
    a_abs = sqrt(a_abs);
    
    b_abs = 0.0;
    for(j=0;j<3;j++){
        b_abs += b[j]*b[j];
    }
    b_abs = sqrt(b_abs);
    //printf("\na_abs = %f b_abs = %f",a_abs,b_abs);
    // if speedup is desired, use cos_theta rather than converting
    // must change threshold value
    cos_theta = dot_prod/(a_abs*b_abs);
    // find projection of b[] on a[] and pass by reference to calling function
    // note scale factor to assure that projection is always less than 256
    *projection = b_abs*cos_theta*255.0/445.0;
    theta = acos(cos_theta);
    //printf("\ntheta = %f", theta);
    return theta;
}

/****************************************************************************/
/* on_error                                                                 */
/* this function handles error messages and exit codes                      */
/****************************************************************************/
void on_error(char message[], int code)
{
    /* send to stdout so messages show up in redirected output files */
    fprintf(stdout, "\n execution error");
    fprintf(stdout, "\n%s, \nexit code %d", message, code);
    /* send to stderr so messages show up on monitor */
    fprintf(stderr, "\n execution error");
    fprintf(stderr, "\n%s, \nexit code %d", message, code);
    exit(code);                 // 1 is abnormal exit
}


