#!/bin/bash
# copyright 2017 Phenotype Screening Corporation, and
# distributed under the terms of the GNU General Public License
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
################################################################################
# convert jpg to raw rgb file using call to ImageMagick
# read rgb image in pixel by pixel and find angle from exemplar color
# write out to 16 bit unsigned int raw file.
# driver for read_txt_leaf software
# exemplar color enter below
# colors must be entered as integer
# set angle theta below in radians, enter as floating point value (with decimal)
# enter filenames without extension, all filenames must have same extension
# set extension with IMG_FORMAT below
# note that size of image is read by call to ImageMagick identify program
# for differrent size image, put in new numbers and recompile
################################################################################

# set RGB values for exemplar color
export RED=60
export GREEN=106
export BLUE=41
# set threshold for projection of pixel value on exemplar
export THRESH=50  	# lowest allowable value for projection on exemplar 
					# eliminate light from shadow regions
echo exemplar color is $RED $GREEN $BLUE threshold is ${THRESH}
# set maximum color divergence from exemplar
# angle is in radians and must be entered as floating point
export ANGLE=0.25
echo ANGLE = $ANGLE
# path to where executables located
export EXE=.
# path to where images are
export DIR=.
# set image format below - this can be automated in further work
# all images must be of same format but dimensions of images can differ
export IMG_FORMAT=.jpg
# insert filenames between for line and do line
# beware of BASH conventions
for BASEFILE in \
57M12001 
do
    export i=$DIR/$BASEFILE
    echo filename is ${i}${IMG_FORMAT}
    # use ImageMagick to find image dimensions
    # note use of ` to left of 1 key and not ' key near enter key
    export IMG_COLS=`identify -format "%w" ${i}${IMG_FORMAT}`
    echo IMG_COLS is $IMG_COLS
    export IMG_ROWS=`identify -format "%h" ${i}${IMG_FORMAT}`
    echo IMG_ROWS is $IMG_ROWS
    # call to ImageMagick convert input file to rgb raw format
    convert ${i}${IMG_FORMAT} -depth 8 ${i}.rgb
    # call PSC executable, reads rgb file and writes 16 bit raw file
    ${EXE}/read_txt_leaf ${RED} ${GREEN} ${BLUE} ${ANGLE} ${THRESH} ${i} ${IMG_COLS} ${IMG_ROWS}
done



